///	ComponentFactory.h - Factory to instantiate all the components in the
///						 ScriptedMainWindow, according to what it reads from
///						 the xml file.
///	--------------------------------------------------------------------------
///	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
///	Copyright (C) 2004-2005  Niall Moody
///	
///	This program is free software; you can redistribute it and/or modify
///	it under the terms of the GNU General Public License as published by
///	the Free Software Foundation; either version 2 of the License, or
///	(at your option) any later version.
///	
///	This program is distributed in the hope that it will be useful,
///	but WITHOUT ANY WARRANTY; without even the implied warranty of
///	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///	GNU General Public License for more details.
///	
///	You should have received a copy of the GNU General Public License
///	along with this program; if not, write to the Free Software
///	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///	
///	--------------------------------------------------------------------------

#ifndef COMPONENTFACTORY_H_
#define COMPONENTFACTORY_H_

#include "FactoryObject.h"

//----------------------------------------------------------------------------
//	Typedef-ing functions to use as function pointers makes things look a lot
//	nicer...
typedef void stringAttributeFunc(String val);
typedef void doubleAttributeFunc(double val);
typedef void intAttributeFunc(int val);
typedef void boolAttributeFunc(bool val);

//----------------------------------------------------------------------------
/// Struct to hold a stringAttributeFunc and it's name.
typedef struct
{
	///	The name of the attribute in the xml file...
	String name;
	///	...which is set from the method pointed to by this pointer.
	stringAttributeFunc *func;
} stringEntry;
/// Struct to hold a doubleAttributeFunc and it's name.
typedef struct
{
	///	The name of the attribute in the xml file...
	String name;
	///	...which is set from the method pointed to by this pointer.
	doubleAttributeFunc *func;
} doubleEntry;
/// Struct to hold an intAttributeFunc and it's name.
typedef struct
{
	///	The name of the attribute in the xml file...
	String name;
	///	...which is set from the method pointed to by this pointer.
	intAttributeFunc *func;
} intEntry;
/// Struct to hold a boolAttributeFunc and it's name.
typedef struct
{
	///	The name of the attribute in the xml file...
	String name;
	///	...which is set from the method pointed to by this pointer.
	boolAttributeFunc *func;
} boolEntry;

//----------------------------------------------------------------------------
/// The ComponentFactory has to keep a note of all the objects it's supposed to support - this is what it stores...
typedef struct
{
	///	The FactoryObject.
	FactoryObject *object;
	///	The name of the component (i.e. the identifier in the xml file).
	String name;
	///	Array of function pointers which set string attributes for the component.
	OwnedArray<stringEntry> stringAttributes;
	///	Array of function pointers which set double attributes for the component.
	OwnedArray<doubleEntry> doubleAttributes;
	///	Array of function pointers which set int attributes for the component.
	OwnedArray<intEntry> intAttributes;
	///	Array of function pointers which set bool attributes for the component.
	OwnedArray<boolEntry> boolAttributes;
} FactoryObjectStuff;

//----------------------------------------------------------------------------
///	Factory object used to instantiate all the components present in your gui.
/*!
	Reads any attributes the components should have from an xml file,
	and sets them in the createComponents() method.

	Components are represented by FactoryObject classes - you should write one
	of these for each component on the gui.  In the constructor of the
	FactoryObject, you register it with the factory (the registerObject()
	method).  You also register all the attributes you want visible to the
	user in the xml file, using the register...Attribute() methods.  With
	these you pass in a function pointer to a \estatic method in the
	FactoryObject which will set the appropriate variables.

	When ComponentFactory::createComponents() is called (remember it must be
	called after the individual components have been constructed - it's the
	constructors where the FactoryObjects register themselves with the
	ComponentFactory), the ComponentFactory will go through the xml file, and
	create the actual components named in the xml file, and set all the
	various attributes for them in turn.

	Note that you don't have to delete any FactoryObject objects - as long as
	you've passed in a valid ComponentFactory pointer to their constructor, 
	the ComponentFactory will delete them in it's destructor.

	\sa FactoryObject
 */
class ComponentFactory
{
  public:
	///	Constructor.
	/*!
		\param window The Component which represents the main window.
		\param filepath The path to the xml file used to instantiate all the 
		Components on the window.
	 */
	ComponentFactory(Component *window, String filepath);
	///	Destructor.
	~ComponentFactory();

	/// Called by each FactoryObject to inform the ComponentFactory it exists.
	void registerObject(FactoryObject *object,
						String& name);
	/// Called by a FactoryObject for every (String) attribute it wants controlled from the xml file (note that method MUST be a static function).
	void registerStringAttribute(FactoryObject *object,
								 String& name,
								 stringAttributeFunc *method);
	/// Called by a FactoryObject for every (double) attribute it wants controlled from the xml file (note that method MUST be a static function).
	void registerDoubleAttribute(FactoryObject *object,
								 String& name,
								 doubleAttributeFunc *method);
	/// Called by a FactoryObject for every (int) attribute it wants controlled from the xml file (note that method MUST be a static function).
	void registerIntAttribute(FactoryObject *object,
							  String& name,
							  intAttributeFunc *method);
	/// Called by a FactoryObject for every (bool) attribute it wants controlled from the xml file (note that method MUST be a static function).
	void registerBoolAttribute(FactoryObject *object,
							   String& name,
							   boolAttributeFunc *method);

	/// Called to read through the xml file and instantiate all the components.
	void createComponents();

	///	Called whenever the parent window is resized, as the components will need resized/positioned too.
	void resizeComponents();

	/// FactoryObject needs to know the window it's parenting it's component to.
	Component *getWindow() {return window;};
  private:
	///	Method used to set the size and position of the object represented by object.
	void setupPosSize(FactoryObjectStuff *object, XmlElement *element);

	///	This is a pointer to the main window/Component.
	Component *window;
	///	The path to the xml file used to instantiate all the gui components.
	String filepath;

	///	An array containing an entry for each FactoryObject registered with the ComponentFactory.
	/*!
		For each FactoryObject, there are OwnedArrays containing function
		pointers to each attribute setting methods in it.
		\sa FactoryObjectStuff
	 */
	OwnedArray<FactoryObjectStuff> array;
};

#endif
