//	NPBPluginFactory.h - Used to simplify adding a new type of plugin
//					     (e.g. NPBPlugin, NPBGain etc.) to the program.
//					     Using this technique means we only need to
//					     update one part of the program (as opposed to
//					     adding a bunch of new ... lines etc.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef NPBPLUGINFACTORY_H_
#define NPBPLUGINFACTORY_H_

#ifndef NPBPLUGINBASE_H_
#include "NPBPluginBase.h"
#endif

///	Abstract base class for the Factory template class.
/*!
	Okay, how this works is FactoryBase provides a method for constructing an
	instance of class Base.  Of course this isn't particularly useful in
	itself, because it would mean we can't store an array of factories.

	\sa Factory
 */
template <class Base> class FactoryBase
{
  public:
	///	Constructor.
	FactoryBase() {};
	///	Destructor.
	virtual ~FactoryBase() {};

	///	Pure virtual function designed to create an instance of class Base, and return a pointer to it.
	virtual Base *createInstance(NPBPluginOrganiser *parent) = 0;
	///	Returns the name of the class this FactoryBase refers to.
	/*!
		(actually it's usually something like "Add a thingummy")
		This is to set up the right-click menu in the NPBPluginOrganiser.
	 */
	String getMenuText() {return MenuText;};
	///	Sets the menu text for the class this FactoryBase refers to.
	/*!
		The menu text is what's displayed by the right-click menu in the
		NPBPluginOrganiser.
	 */
	void setMenuText(String text) {MenuText = text;};
  private:
	/*!
		The menu text is what's displayed by the right-click menu in the
		NPBPluginOrganiser.
	 */
	String MenuText;
};

///	The template class we use for the factories for our NPBPluginBase subclasses.
/*!
	So the reasoning behind this is we have a Factory object which is used
	to create an instance of one of our NPBPluginBase subclasses.  What's
	complicated about it is that we want to be able to refer to each factory
	as if it's essentially the same thing (we want to store an array of
	factories, for instance).  This is not possible however, as each factory
	will have a different template member.

	The solution is that we have a FactoryBase base class, which Factory
	inherits from.  In this way we can store an array of FactoryBases, and
	access them through polymorhpism. For instance:
	\code
	//Set up a factory for the ExamplePlugin class, which inherits from NPBPluginBase.
	Factory<NPBPluginBase, ExamplePlugin> exampleFactory;
	//exampleFactory will now be able to create an instance of ExamplePlugin, with the createInstance() method.

	//We can also add it to an array of FactoryBases:
	//(note we've typedef-ed FactoryBase<NPBPluginBase> as NPBPluginBaseFactory)
	Array<NPBPluginBaseFactory *> exampleFactory;
	exampleFactory.add(&exampleFactory);
	//...and add some text for the menu entry:
	exampleFactory[0]->setMenuText("Add an ExamplePlugin");
	\endcode

	\sa FactoryBase
 */
template <class Base, class Child> class Factory : public FactoryBase<Base>
{
  public:
	///	Constructor.
	Factory() {};
	///	Destructor.
	~Factory() {};

	///	Returns an instance of Child, which must be a subclass of Base.
	Base *createInstance(NPBPluginOrganiser *parent) {return new Child(parent);};
};

///	For convenience, allows us to use NPBPluginBaseFactory as a template parameter itself.
/*!
	(it's just more portable this way)
 */
typedef FactoryBase<class NPBPluginBase> NPBPluginBaseFactory;

#endif
