//	NPBPluginGUI.h - Editor dialog for plugins which supply their own
//				     gui.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef NPBPLUGINGUI_H_
#define NPBPLUGINGUI_H_

class NPBPluginGUI;
class NPBPluginGUIContent;

#ifndef NPBPLUGIN_H_
#include "NPBPlugin.h"
#endif


///	The actual window provided for the VST plugin's editor.
/*!	
	This is the window that holds the plugin's gui.  Note that it is
	a separate component from NPBPluginGUI - this is because
	AEffEditor::open() requires an HWND (on Windows).  In JUCE you
	can only get at that if your component is on the desktop, so we
	create two desktop components, and pretend that NPBPluginGUI
	controls EditorComponent (sets it's position etc.).
 */
class EditorComponent : public DialogWindow, //I don't know why, but the plugin's editor won't show if this isn't a subclass of DialogWindow
							   Timer
{
  public:
	///	Constructor.
	/*!
		\param plugin The VST plugin whose editor is drawn in this window.
		\param parent The window we pretend/fake is the parent to this window.
		\param col The background colour to paint the window (surely this
		isn't necessary?
	 */
	EditorComponent(AEffect *plugin, NPBPluginGUI *parent, Colour col);
	///	Destructor.
	~EditorComponent();

	///	Used to provide idle() calls for the plugin's editor.
	void timerCallback();
  private:
	AEffect *Plugin;
	NPBPluginGUI *Parent;
};

///	The main DialogWindow for a plugin's editor.
/*!
	Basically just provides a title bar and some controls (program, mix...)
	for the editor.

	\sa EditorComponent
 */
class NPBPluginGUI : public DialogWindow
{
  public:
	///	Constructor.
	/*!
		\param plugin The VST plugin whose editor is drawn in this window.
		\param parent The window we pretend/fake is the parent to this window.
		\param col The background colour to paint the window.
	 */
	NPBPluginGUI(AEffect *plugin, NPBPlugin *parent, Colour col);
	///	Destructor.
	~NPBPluginGUI();

	///	Called when the window has changed position.
	/*!
		We use this to inform the plugin's actual editor window that we've
		moved, so it can move too, in conjunction with this window.
	 */
	void moved() {if(ActualEditor){ActualEditor->toFront(false); ActualEditor->setTopLeftPosition((getX()+6), (getY()+58));}};
	///	Sets the size of the window.
	/*!
		This is called from EditorComponent, once we've determined the size
		of the plugin's editor, so we can resize this window accordingly.
	 */
	void setSize(const int newWidth, const int newHeight);

	///	Brings the plugin's editor (EditorComponent) to front.
	/*!	
		The plugin's editor has a habit of disappearing behind this window,
		so we have to do this after certain events (program selection etc.).
	 */
	void updateEditor() {if(ActualEditor) ActualEditor->toFront(false);};

	///	Sets the value of the Mic slider.
	void setMix(float val);

	///	Called when the window's resized.
	/*!
		We override this to make sure the EditorComponent is brought to the
		front.

		\sa updateEditor()
	 */
	void resized();
  private:
	NPBPlugin *Parent;
	EditorComponent *ActualEditor;
};

///	The content component for NPBPluginGUI.
/*!
	Implements controls to select the plugin's program, and a mix Slider.
 */
class NPBPluginGUIContent : public Component,
								   ActionListener,
								   ChangeListener
{
  public:
	///	Constructor.
	/*!
		\param plugin Pointer to the actual VST plugin.
		\param parent Pointer to our NPBPlugin containing the VST plugin.
		\param win Pointer to the DialogWindow this Component is contained
		within.
	 */
	NPBPluginGUIContent(AEffect *plugin, NPBPlugin *parent, NPBPluginGUI *win);
	///	Destructor.
	~NPBPluginGUIContent();

	///	Called when the user selects a program.
	void actionListenerCallback(const String& m);
	///	Called when the Mix Slider is moved.
	void changeListenerCallback(void *objectThatHasChanged);

	///	Huh?
	int getWidth2() {return Width;};
	///	Huh?
	int getHeight2() {return Height;};

	///	Sets the value of the Mix Slider.
	void setMix(float val) {Mix->setValue((double)val);};

	///	Called when the component is resized.
	/*!
		This is usually only just after we've determined the correct size of
		the VST plugin's editor.
	 */
	void resized();
  protected:
	AEffect *Plugin;
	NPBPlugin *Parent;
	NPBPluginGUI *Win; //So we can resize the window according to the plugin's gui.

	ComboBox *Programs;
	Label *MixLabel;
	Slider *Mix;
	Component *GUI;

	int Width, Height;
};

#endif
